@@ -44,6 +44,41 @@ fn optionals() -> Result<()> {
     Ok(())
 }
 
+#[test]
+fn ensuredir() -> Result<()> {
+    let td = cap_tempfile::tempdir(cap_std::ambient_authority())?;
+
+    let p = Path::new("somedir");
+    let b = &cap_std::fs::DirBuilder::new();
+    assert!(td.metadata_optional(p)?.is_none());
+    assert!(td.ensure_dir_with(p, b).unwrap());
+    assert!(td.metadata_optional(p)?.is_some());
+    assert_eq!(td.ensure_dir_with(p, b).unwrap(), false);
+
+    let p = Path::new("somedir/without/existing-parent");
+    // We should fail because the intermediate directory doesn't exist.
+    assert!(td.ensure_dir_with(p, b).is_err());
+    // Now create the parent
+    assert!(td.ensure_dir_with(p.parent().unwrap(), b).unwrap());
+    assert!(td.ensure_dir_with(p, b).unwrap());
+    assert_eq!(td.ensure_dir_with(p, b).unwrap(), false);
+
+    // Verify we don't replace a file
+    let p = Path::new("somefile");
+    td.write(p, "some file contents")?;
+    assert!(td.ensure_dir_with(p, b).is_err());
+
+    // Broken symlinks aren't followed and are errors
+    let p = Path::new("linksrc");
+    td.symlink("linkdest", p)?;
+    // Non-broken symlinks are also an error
+    assert!(td.ensure_dir_with(p, b).is_err());
+    td.create_dir("linkdest")?;
+    assert!(td.ensure_dir_with(p, b).is_err());
+
+    Ok(())
+}
+
 #[test]
 fn link_tempfile() -> Result<()> {
     let td = cap_tempfile::tempdir(cap_std::ambient_authority())?;
